

#include <p18f27j53.h>
#include "fuses.h"
#include <usart.h>
#include <stdio.h>
#include <delays.h>


void Delay10ms(unsigned char x)
{
	do {
		Delay1KTCYx(60);
	} while(--x);
}

void Test(void)
{
	const unsigned char period = 240;	// 24000000 / 4 / 240 = 25 kHz
	unsigned char x;
	unsigned char n;
	unsigned adc;
	float pwm;
	float volts;
	float R2 = 10.0;
	float R3 = 2.2;
	float vref = 2.5;

	ODCON1bits.CCP4OD = 1;		// Open drain output

	T4CONbits.T4OUTPS = 0;		// 1:1 postscale
	T4CONbits.T4CKPS = 0;		// 1:1 prescale
	T4CONbits.TMR4ON = 1;		// TMR4 on
	PR4 = period;				// TRM4/CCP period

	CCP4CONbits.CCP4M = 0x0C;	// PWM mode
	CCPTMRS1bits.C4TSEL = 1;	// Use TMR4/PR4
	
	//CCPR4L = 168; while(1);
	
	for(x = 0; x <= period; ++x) {
		CCPR4L = x;	

		Delay10ms(100);

		n = 16;
		adc = 0;
		do {
			ADCON0bits.GO = 1;
			while(ADCON0bits.GO);
			adc += ((unsigned)ADRESH << 8 | ADRESL);
			Delay1KTCYx(1);
		} while(--n);
		adc /= 16;

		pwm = (float)x * 100.0 / period;
		volts = (float)adc * vref * (R2 + R3) / (R3 * 4095.0);

		//printf((const char rom far *)"PWM %i.%02u%%  %i.%02u volts\r\n", 
		printf((const char rom far *)"%i.%02u, %i.%02u\r\n", 
			(int)pwm , ((int) (((pwm)-(int)pwm) * 100)),
			(int)volts , ((int) (((volts)-(int)volts) * 100)));
	}
}

void main(void)
{
	ANCON0 = 0xFF ^ 0x1F;				// AN0, AN1, AN2, AN3 and AN4 (RA5) as analog
	ANCON1 = 0xFF ^ 0x01;				// AN8 (RB2) as analog

	TRISA = 0xEF;
	LATA = 0;

	TRISB = 0x04;
	LATB = 0;

	TRISC = 0x80;
	LATC = 0x40;

	ADCON0bits.VCFG0 = 1;				// Vref = AN3
	ADCON1bits.ADFM = 1;				// Right justified result
	ADCON1bits.ADCS = 6;				// Fosc/64 conversion clock
	ADCON0bits.CHS = 8;					// Channel 8
	ADCON0bits.ADON = 1;				// Turn on ADC


	Open1USART(USART_TX_INT_OFF & USART_RX_INT_OFF & USART_ASYNCH_MODE &\
		USART_EIGHT_BIT & USART_CONT_RX & USART_BRGH_HIGH,
		( 24000000 / 16 / 38400 ) - 1);

	//printf((const char rom far *)"\r\nFan Test\r\n");

	//while(1)
		Test();

	while(1);
}
